/*
 *
 * Simple wait and delay routines using Timer 2
 * 1 ms 7-segment display multiplexer
 * Display primitives
 *
 * Juha Niinikoski 7.4.2003
 *
 * Project switched to PIC16F818 04.09.2003
 * TMR0 frequency corrected 05.09.2003
 *
 * Character generator for Mitsu hardware 31.01.2005
 * Comparator interrupts added / tachometer board 11.03.2005
 * Changed to HDSP-B03E display & new PCB 30.03.2005
 *
 * Revctr_3 code
 * Frequency counter type measurement 07.05.2005
 *
*/

// delays calculated for 4 MHz clock

/* wait counters */

static near ms_counter;		// 1 second counter
static near int msw;		// ms wait counter

/* timer1 variables */
union
	{
	unsigned int r;
	unsigned char tmr1regs[2];
	} tmr1;


/* Character generator & display variables */

// define 7-segment numbers & some characters

#define C_0	(A+B+C+D+E+F)
#define C_1	(B+C)
#define C_2	(A+B+G+E+D)
#define C_3	(A+B+C+D+G)
#define C_4	(F+G+B+C)
#define C_5	(A+F+G+C+D)
#define C_6	(A+F+E+D+C+G)
#define C_7	(A+B+C)
#define C_8	(A+B+C+D+E+F+G)
#define C_9	(A+B+C+D+F+G)

#define C_NEG (G)
#define C_o (G+C+D+E)
#define C_u (C+D+E)
#define C_i (C)
#define C_n (E+G+C)

#define C_J (B+C+D+E)
#define C_U (B+C+D+E+F)
#define C_H (B+C+E+F+G)
#define C_A (A+B+C+E+F+G)

static near unsigned char digit_ctr;		// digit counter

const unsigned char chgen[32] =
{C_0, C_1, C_2, C_3, C_4, C_5, C_6, C_7, C_8, C_9, C_NEG, C_i, C_o,  C_NEG, Dp, 0,
C_u, C_n, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,};

const unsigned char digit_bits[4] = {0x80, 0x40, 0x08, 0x10};	// digit select bits


/* Init TMR2 for 1 mS interrupt generation & Comparator irq */

void init_timers(void)
	{
unsigned char dummy;


	digit_ctr = 0;
	s0 = 0;
	ms_counter = 1000;			// init counters
	spark_tmr1 = SPARK_GATE/2;	// init gate timers, first with half cycle
	spark_tmr2 = SPARK_GATE;	// init gate timers, second with full

	OPTION = (OPTION & 0xC1);  /* 250kHz clock for TMR0 */

	T1CKPS0 = 1;	// 8us clock for TMR1
	T1CKPS1 = 1;
	T1SYNC = 1;		// no sync, internal clock
	TMR1ON = 1;		// start timer 1

	TMR2ON = 1;		// start TIMER2
	PR2 = 249;		// set TMR2 cycle time 1mS
	T2CKPS0 = 1;	// Prescaler = 4 = 250kHz clock for TMR2
	TMR2IE = 1;		// enable TMR2 IRQ

	dummy = CMCON;	// reset possible missmatch condition
	CMIF = 0;
	CMIE = 1;		// enable comparator IRQ

	PEIE = 1;		// enable peripheral IRQs
	GIE = 1;		// enable interrupts
	}


/* Display limited range  uint on LED display */

void putintdec( unsigned int c)
{
	if(c > 9999)
		c = 9999;		// limit renge & digit values to max

	//thousands
	digit_data[3] = chgen[(c/1000)];
	c-=(c/1000)*1000;

	//hunreds
	digit_data[2] = chgen[(c/100)];
	c-=(c/100)*100;

	//tens
	digit_data[1] = chgen[(c/10)];
	c-=(c/10)*10;

	//ones
	digit_data[0] = chgen[(c/1)];

	//add marker points to display
	digit_data[1] = digit_data[1] | dp_data[1];
	digit_data[2] = digit_data[2] | dp_data[2];
}


/* Display therr byte long string on LED display */

void putst(register const char *str)
{
	digit_data[3] = chgen[(*str & 0x1F)]; 
	str++;
	digit_data[2] = chgen[(*str & 0x1F)]; 
	str++;
	digit_data[1] = chgen[(*str & 0x1F)]; 
	str++;
	digit_data[0] = chgen[(*str & 0x1F)]; 
}


/* Millisecond delays */

// note! msw is also used for certain timeouts

void delayms( int millisec )
	{
	msw = millisec; 
	while(msw != 0)
		;
	}


/* comparator IRQ,  exec about 20us */
/* TMR1 clock = 8us */

void compint(void)
{
unsigned char tmp;
	tmp = CMCON;					// save & clear comparator status
	CMIF = 0;

	if(tmp & SIGNAL)				// signal comparator logic
		{
		TMR1ON = 0;					// stop timer
 		strobo = SB;				// Strobo LED on
		DIGIT_DRIVE = digit_sel | SB; // and maintain digit drive bits

		if(TMR1IF)					// check overflow
			{
			tmr1.tmr1regs[0] = 255;	// set to max count
			tmr1.tmr1regs[0] = 255;	// set to max count
			TMR1IF = 0;				// reset overflow condition
			}
		else
			{
			tmr1.tmr1regs[0] = TMR1L;	// save result
			tmr1.tmr1regs[1] = TMR1H;
			}

		if(tmr1.r > glitch)			// test if under xx ms, glitch filter
			{
			TMR1L = 0;				// reset timer
			TMR1H = 0;
			t1_result = tmr1.r;		// save result
			spark_ctr1++;			// count sparks
			spark_ctr2++;			// count sparks
			meas_ready = 1;			// set measurement ready flag
			}

		TMR1ON = 1;					// restart counter timer

		TMR0 = 0;					// reset strobo timer, 1,024 ms flash
		T0IF = 0;					// reset possible pending irq
		T0IE = 1;					// arm strobo off irq
		}

	if(tmp & MODE)					// mode comparator logic
		button = 0;
	else
		button = 1;
}

/* IRQ handler */

static void interrupt genint()
{
	if(CMIF)						// comparator irq
		{
		compint();
		}

	if(TMR2IF)						// 1ms tick irq
		{
		TMR2IF = 0;			// reset IRQ flag

		// uppdate display every millisecond
		digit_ctr ++;				// uppdate digit counter
		digit_ctr = digit_ctr & 3;	// modulo 4 counter, 4 digit display

		SEGMENT_DRIVE = 0;			// shut display off

		digit_sel = digit_bits[digit_ctr];		// new digit drive bits
		DIGIT_DRIVE = digit_sel | strobo;		// set new digit drive & maintain strobo state
			
		SEGMENT_DRIVE = digit_data[digit_ctr];	// set new segment bits / displat on

		if(msw != 0)			// decrement ms delay & wait counter
			{
			msw--;
			}

		spark_tmr1--;					// 0 deg
		if(spark_tmr1 == 0)
			{
			spark_tmr1 = SPARK_GATE;	// reload gate timer
			spark_count = spark_ctr1;	// save spark counter
			spark_ctr1 = 0;				// reset spark counter
			spark_ready = 1;			// result ready, counter 1	
			}

		spark_tmr2--;					// 180 deg
		if(spark_tmr2 == 0)
			{
			spark_tmr2 = SPARK_GATE;		// reload gate timer
			spark_count = spark_ctr2;	// save spark counter
			spark_ctr2 = 0;				// reset spark counter
			spark_ready = 1;			// result ready, counter 2	
			}
		}

	if(T0IF && T0IE)
		{
		T0IE = 0;					// stop strobo irq
 		strobo = 0;					// Strobo LED off
		DIGIT_DRIVE = digit_sel;	// maintain digit select bits
		}
}
