/*
 *
 * Tachometer with HDSP-B03E display ver 1.0
 * MPLAB 7.0 / Hitec PICC 8.05PL2 compiler
 *
 * Juha Niinikoski 11.03.2005
 * 
 * Average filter added 13.03.2005
 * Changed to HDSP-B03E display & new PCB 30.03.2005
 * Unique calibration values for two first units 31.03.2005
 *
 * Revctr_3 code
 * Frequency counter type measurement 07.05.2005
 * Comparator ref voltage dropped to 200mV 12.05.2005, v3.1
 *
*/

#include <pic.h>
#include "revctr3.h"

/* global variables */

near bit s_flag;				// second elapsed
near unsigned char s0;			// second wait counters

static near unsigned char digit_data[4];	// segment bits
static near unsigned char dp_data[4];		// DP bits
static near unsigned char digit_sel;		// to maintain PORTA bits
static near unsigned char strobo;

bit run_flag;					// signal ok flag
bit mode;						// operating mode, 1 = waste spark
bit button;						// button pressed from comparator

bit meas_ready;					// measurement ready
unsigned int t1_result;			// timer1 result
unsigned int rpm;				// rpm

#define NUM_AVG 4				// average size
unsigned int avg[NUM_AVG];		// average array
char idx_avg;					// array index
unsigned int rpm_avg;			// averaged rpm

// nominal values / calibration
// for accurate display 16F628 internal oscillator
// may require unique calibration

// Chip 02461GC
//#warning Cal 02461GC / bo (1,029)used
//#define two_stroke   7717500UL

// Chip 03362UM / 1 (1,033)
//#warning Cal 03362UM / 1 used
//#define two_stroke   7747500UL

// Chip 03362UM / 2
//#warning Cal 03362UM / 2 used
//#define two_stroke   7765315UL

 
#define two_stroke   7500000UL	// constant for two stroke or waste spark 4 stroke engine, spark at every round
#define four_stroke (two_stroke * 2)	// constant for four stroke engine, two cranc rounds / spark

// filter resolution 8us/step
//#define glitch	127				// test if new trigger closer than 1,024ms, glitch filter (original)			
#define glitch	750				// test if new trigger closer than 6ms, glitch filter 6.4.2005	
//#define glitch	1000		// test 8ms

// Equation for two stroke (spark/round)
//
// 1 rpm = 60s, TMR1 clock = 8us
// 60s/8us = 7.500.000
// RPM Display = 7.500.000 / TMR1count
// 
// TMR1 max count = 65535, min RPM display = 114 RPM
//
// Four stroke (spark at every second roune) 
//
// Constant = 2 * 7.500.000
// Min RPM display = 228 RPM
//
// MAX RPM display limited by display to 9999

// frequency counter methode variables 07.05.2005

// calibration values for diggerent chips

// Chip 03362UM / 1
//#warning Cal 03362UM / 1 used (1,034)
//#define SPARK_GATE 6206			// spark counter gate timer, should be even number

// Chip 02461GC
//#warning Cal 02461GC / bo (1,029)used
//#define SPARK_GATE 6174			// spark counter gate timer, should be even number

// nominal values for 10 RPM resolution, slow
#define SPARK_GATE 6000			// spark counter gate timer (6000 ms)
#define SPARK_MULT 10			// RPM = spark_count * SPARK_MULT

// nominal values for 50 RPM resolution, fast
//#define SPARK_GATE 1200			// spark counter gate timer (1200 ms)
//#define SPARK_MULT 50			// RPM = spark_count * SPARK_MULT

unsigned int spark_ctr1;		// spark counter 1
unsigned int spark_ctr2;		// spark counter 2
unsigned int spark_tmr1;		// spark counter gate timer (ms), 0deg
unsigned int spark_tmr2;		// spark counter gate timer (ms), 180deg

unsigned int spark_count;		// spark count result
bit spark_ready;				// counting cycle ready



/* included programs */

#include "7segmux3.c"

/* processor config & id info */

	__CONFIG(UNPROTECT & INTIO & WDTDIS & PWRTEN & BOREN & LVPDIS & MCLRDIS);	// 16F628 & 16F81x

	__IDLOC(0103);							// Device ID

const char id[] = {"Juha Niinikoski 07.04.2005"};	// Visible id string

/* main  */

void main(void)
{
char x;

	CMCON = CMCON_INIT;			// internal ref, inverted, CIS = 0
	VRCON = VRCON_INIT;			// voltage reference init

	PORTB = PORTB_INIT;
	PORTA = PORTA_INIT;

	TRISB = TRISB_INIT;			// RB5 - RB7 otputs
	TRISA = TRISA_INIT;

	init_timers();				// start timing functions

// display name
	digit_data[3] = C_J;		// display start up message
	digit_data[2] = C_U;	
	digit_data[1] = C_H;
	digit_data[0] = C_A;
	delayms(1500);

// display version
	digit_data[3] = C_NEG;	
	digit_data[2] = C_3;		// version number 3.1
	digit_data[1] = C_1;
	digit_data[0] = C_NEG;
	delayms(1500);

	for(;;)
		{

// period measurement
// if measurement ready calculate rpm
		if(meas_ready)
			{
			meas_ready = 0;				// reset ready flag
			msw = 1000;					// reset timeout

// select only one methode
#if 0
			run_flag = !run_flag;		// toggle display run flag

			if(mode)					// print result, spark / two rounds
				{
				rpm = four_stroke / t1_result;
				}
			else						// spark / round
				{
				rpm = two_stroke / t1_result;
				}
# endif
			}

// frequency counter methode
// if gate timer calculate rpm
		if(spark_ready)
			{
			spark_ready = 0;
			run_flag = !run_flag;		// toggle display run flag
	

			if(mode)					// print result, spark / two rounds (four stroke)
				{
				rpm = spark_count * (SPARK_MULT * 2);
				}
			else						// spark / round (two stroke)
				{
				rpm = spark_count * SPARK_MULT;
				}
			}

// dispaly rpm value

// average filter for result
			avg[idx_avg] = rpm;			// store last result
			idx_avg++;					// addjust index
			if(idx_avg >= NUM_AVG)
				idx_avg = 0;

			rpm_avg = 0;				// calculate average
			for(x=0; x< NUM_AVG; x++)
				{
				rpm_avg = rpm_avg + avg[x];
				}

			rpm_avg = rpm_avg + (NUM_AVG / 2);	// round
			rpm_avg = rpm_avg / NUM_AVG;

// display average
//			putintdec(rpm_avg);

// no averaging
			putintdec(rpm);


// slow signal trap, use only with period measurement
#if 0
		if(msw == 0)					// note msw is also used for ms delay
				{
				putst("-???");			// no signal display, save power
				run_flag = 0;			// blanc point, save power
				}
#endif

// set DP point flags for display

		if (run_flag == 1)				// show run flag
			dp_data[2] = Dp;
		else
			dp_data[2] = 0;

		if (mode == 1)
			dp_data[1] = Dp;			// mode point (4 stroke select) lower dot
		else
			dp_data[1] = 0;

// evaluate mode change button
		if (button == 1)
			{
			mode = !mode;				// toggle mode
			delayms(400);				// debounce
			}

		}

}
